@@ -696,10 +696,6 @@ _orig_default_visit = default_visit
 
 def default_enter(path, key, value):
     # print('enter(%r, %r)' % (key, value))
-    try:
-        iter(value)
-    except TypeError:
-        return value, False
     if isinstance(value, basestring):
         return value, False
     elif isinstance(value, Mapping):
@@ -708,7 +704,10 @@ def default_enter(path, key, value):
         return value.__class__(), enumerate(value)
     elif isinstance(value, Set):
         return value.__class__(), enumerate(value)
-    return value, False
+    else:
+        # files, strings, other iterables, and scalars are not
+        # traversed
+        return value, False
 
 
 def default_exit(path, key, old_parent, new_parent, new_items):
@@ -902,8 +901,10 @@ def remap(root, visit=default_visit, enter=default_enter, exit=default_exit,
 
 
 class PathAccessError(KeyError, IndexError, TypeError):
-    # TODO: could maybe get fancy with an isinstance
-    # TODO: should accept an idx argument
+    """An amalgamation of KeyError, IndexError, and TypeError,
+    representing what can occur when looking up a path in a nested
+    object.
+    """
     def __init__(self, exc, seg, path):
         self.exc = exc
         self.seg = seg
@@ -919,42 +920,131 @@ class PathAccessError(KeyError, IndexError, TypeError):
 
 
 def get_path(root, path, default=_UNSET):
-    """EAFP is great, but the error message on this isn't:
+    """Retrieve a value from a nested object via a tuple representing the
+    lookup path.
 
-    var_key = 'last_key'
-    x['key'][-1]['other_key'][var_key]
-    KeyError: 'last_key'
+    >>> root = {'a': {'b': {'c': [[1], [2], [3]]}}}
+    >>> get_path(root, ('a', 'b', 'c', 2, 0))
+    3
 
-    One of get_path's chief aims is to have a good exception that is
-    better than a plain old KeyError: 'missing_key'
+    The path format is intentionally consistent with that of
+    :func:`remap`.
+
+    One of get_path's chief aims is improved error messaging. EAFP is
+    great, but the error messages are not.
+
+    For instance, ``root['a']['b']['c'][2][1]`` gives back
+    ``IndexError: list index out of range``
+
+    What went out of range where? get_path currently raises
+    ``PathAccessError: could not access 2 from path ('a', 'b', 'c', 2,
+    1), got error: IndexError('list index out of range',)``, a
+    subclass of IndexError and KeyError.
+
+    You can also pass a default that covers the entire operation,
+    should the lookup fail at any level.
+
+    Args:
+       root: The target nesting of dictionaries, lists, or other
+          objects supporting ``__getitem__``.
+       path (tuple): A list of strings and integers to be successively
+          looked up within *root*.
+       default: The value to be returned should any
+          ``PathAccessError`` exceptions be raised.
     """
-    # TODO: integrate default
-    # TODO: listify kwarg? to allow indexing into sets
-    # TODO: raise better error on not iterable?
     if isinstance(path, basestring):
         path = path.split('.')
     cur = root
-    for seg in path:
-        try:
-            cur = cur[seg]
-        except (KeyError, IndexError) as exc:
-            raise PathAccessError(exc, seg, path)
-        except TypeError as exc:
-            # either string index in a list, or a parent that
-            # doesn't support indexing
+    try:
+        for seg in path:
             try:
-                seg = int(seg)
                 cur = cur[seg]
-            except (ValueError, KeyError, IndexError, TypeError):
+            except (KeyError, IndexError) as exc:
                 raise PathAccessError(exc, seg, path)
+            except TypeError as exc:
+                # either string index in a list, or a parent that
+                # doesn't support indexing
+                try:
+                    seg = int(seg)
+                    cur = cur[seg]
+                except (ValueError, KeyError, IndexError, TypeError):
+                    if not is_iterable(cur):
+                        exc = TypeError('%r object is not indexable'
+                                        % type(cur).__name__)
+                    raise PathAccessError(exc, seg, path)
+    except PathAccessError:
+        if default is _UNSET:
+            raise
+        return default
     return cur
 
-# TODO: get_path/set_path
+
+def research(root, query=lambda p, k, v: True, reraise=False):
+    """The :func:`research` function uses :func:`remap` to recurse over
+    any data nested in *root*, and find values which match a given
+    criterion, specified by the *query* callable.
+
+    Results are returned as a list of ``(path, value)`` pairs. The
+    paths are tuples in the same format accepted by
+    :func:`get_path`. This can be useful for comparing values nested
+    in two or more different structures.
+
+    Here's a simple example that finds all integers:
+
+    >>> root = {'a': {'b': 1, 'c': (2, 'd', 3)}, 'e': None}
+    >>> res = research(root, query=lambda p, k, v: isinstance(v, int))
+    >>> print(sorted(res))
+    [(('a', 'b'), 1), (('a', 'c', 0), 2), (('a', 'c', 2), 3)]
+
+    Note how *query* follows the same, familiar ``path, key, value``
+    signature as the ``visit`` and ``enter`` functions on
+    :func:`remap`, and returns a :class:`bool`.
+
+    Args:
+       root: The target object to search. Supports the same types of
+          objects as :func:`remap`, including :class:`list`,
+          :class:`tuple`, :class:`dict`, and :class:`set`.
+       query (callable): The function called on every object to
+          determine whether to include it in the search results. The
+          callable must accept three arguments, *path*, *key*, and
+          *value*, commonly abbreviated *p*, *k*, and *v*, same as
+          *enter* and *visit* from :func:`remap`.
+       reraise (bool): Whether to reraise exceptions raised by *query*
+          or to simply drop the result that caused the error.
+
+
+    With :func:`research` it's easy to inspect the details of a data
+    structure, like finding values that are at a certain depth (using
+    ``len(p)``) and much more. If more advanced functionality is
+    needed, check out the code and make your own :func:`remap`
+    wrapper, and consider `submitting a patch`_!
+
+    .. _submitting a patch: https://github.com/mahmoud/boltons/pulls
+    """
+    ret = []
+
+    if not callable(query):
+        raise TypeError('query expected callable, not: %r' % query)
+
+    def enter(path, key, value):
+        try:
+            if query(path, key, value):
+                ret.append((path + (key,), value))
+        except Exception:
+            if reraise:
+                raise
+        return default_enter(path, key, value)
+
+    remap(root, enter=enter)
+    return ret
+
+
 # TODO: recollect()
+# TODO: refilter()
 # TODO: reiter()
 
-# GUID iterators: 10x faster and somewhat more compact than uuid.
 
+# GUID iterators: 10x faster and somewhat more compact than uuid.
 
 class GUIDerator(object):
     """The GUIDerator is an iterator that yields a globally-unique
@@ -963,22 +1053,23 @@ class GUIDerator(object):
 
     Testing shows it to be around 12x faster than the uuid module. By
     default it is also more compact, partly due to its default 96-bit
-    (12-byte) length. 96 bits of randomness means that there is a 1 in
-    2 ^ 32 chance of collision after 2 ^ 64 iterations. If more or
-    less uniqueness is desired, the *size* argument can be adjusted
+    (24-hexdigit) length. 96 bits of randomness means that there is a
+    1 in 2 ^ 32 chance of collision after 2 ^ 64 iterations. If more
+    or less uniqueness is desired, the *size* argument can be adjusted
     accordingly.
 
     Args:
-        size (int): character length of the GUID, defaults to 12. Lengths
-                    between 10 and 20 are considered valid.
+        size (int): character length of the GUID, defaults to 24. Lengths
+                    between 20 and 36 are considered valid.
 
     The GUIDerator has built-in fork protection that causes it to
     detect a fork on next iteration and reseed accordingly.
+
     """
-    def __init__(self, size=12):
+    def __init__(self, size=24):
         self.size = size
-        if size < 10 or size > 20:
-            raise ValueError('expected 10 < size <= 20')
+        if size < 20 or size > 36:
+            raise ValueError('expected 20 < size <= 36')
         self.count = itertools.count()
         self.reseed()
 
@@ -1025,13 +1116,13 @@ class SequentialGUIDerator(GUIDerator):
     The SequentialGUIDerator is aronud 50% faster than the normal
     GUIDerator, making it almost 20x as fast as the built-in uuid
     module. By default it is also more compact, partly due to its
-    96-bit (12-byte) default length. 96 bits of randomness means that
+    96-bit (24-hexdigit) default length. 96 bits of randomness means that
     there is a 1 in 2 ^ 32 chance of collision after 2 ^ 64
     iterations. If more or less uniqueness is desired, the *size*
     argument can be adjusted accordingly.
 
     Args:
-        size (int): character length of the GUID, defaults to 12.
+        size (int): character length of the GUID, defaults to 24.
 
     Note that with SequentialGUIDerator there is a chance of GUIDs
     growing larger than the size configured. The SequentialGUIDerator
