@@ -972,3 +972,117 @@ def format_int_list(int_list, delim=',', range_delim='-', delim_space=False):
         output_str = delim.join(output)
 
     return output_str
+
+
+class MultiReplace(object):
+    """
+    MultiReplace is a tool for doing multiple find/replace actions in one pass.
+
+    Given a mapping of values to be replaced it allows for all of the matching
+    values to be replaced in a single pass which can save a lot of performance
+    on very large strings. In addition to simple replace, it also allows for
+    replacing based on regular expressions.
+
+    Keyword Arguments:
+
+    :type regex: bool
+    :param regex: Treat search keys as regular expressions [Default: False]
+    :type flags: int
+    :param flags: flags to pass to the regex engine during compile
+
+    Dictionary Usage::
+
+        from lrmslib import stringutils
+        s = stringutils.MultiReplace({
+            'foo': 'zoo',
+            'cat': 'hat',
+            'bat': 'kraken'
+        })
+        new = s.sub('The foo bar cat ate a bat')
+        new == 'The zoo bar hat ate a kraken'
+
+    Iterable Usage::
+
+        from lrmslib import stringutils
+        s = stringutils.MultiReplace([
+            ('foo', 'zoo'),
+            ('cat', 'hat'),
+            ('bat', 'kraken)'
+        ])
+        new = s.sub('The foo bar cat ate a bat')
+        new == 'The zoo bar hat ate a kraken'
+
+
+    The constructor can be passed a dictionary or other mapping as well as
+    an iterable of tuples. If given an iterable, the substitution will be run
+    in the order the replacement values are specified in the iterable. This is
+    also true if it is given an OrderedDict. If given a dictionary then the
+    order will be non-deterministic::
+
+        >>> 'foo bar baz'.replace('foo', 'baz').replace('baz', 'bar')
+        'bar bar bar'
+        >>> m = MultiReplace({'foo': 'baz', 'baz': 'bar'})
+        >>> m.sub('foo bar baz')
+        'baz bar bar'
+
+    This is because the order of replacement can matter if you're inserting
+    something that might be replaced by a later substitution. Pay attention and
+    if you need to rely on order then consider using a list of tuples instead
+    of a dictionary.
+    """
+
+    def __init__(self, sub_map, **kwargs):
+        """Compile any regular expressions that have been passed."""
+        options = {
+            'regex': False,
+            'flags': 0,
+        }
+        options.update(kwargs)
+        self.group_map = {}
+        regex_values = []
+
+        if isinstance(sub_map, collections.Mapping):
+            sub_map = sub_map.items()
+
+        for idx, vals in enumerate(sub_map):
+            group_name = 'group{0}'.format(idx)
+            if isinstance(vals[0], basestring):
+                # If we're not treating input strings like a regex, escape it
+                if not options['regex']:
+                    exp = re.escape(vals[0])
+                else:
+                    exp = vals[0]
+            else:
+                exp = vals[0].pattern
+
+            regex_values.append('(?P<{0}>{1})'.format(
+                group_name,
+                exp
+            ))
+            self.group_map[group_name] = vals[1]
+
+        self.combined_pattern = re.compile(
+            '|'.join(regex_values),
+            flags=options['flags']
+        )
+
+    def _get_value(self, match):
+        """Given a match object find replacement value."""
+        group_dict = match.groupdict()
+        key = [x for x in group_dict if group_dict[x]][0]
+        return self.group_map[key]
+
+    def sub(self, text):
+        """
+        Run substitutions on the input text.
+
+        Given an input string, run all substitutions given in the
+        constructor.
+        """
+        return self.combined_pattern.sub(self._get_value, text)
+
+
+def multi_replace(text, sub_map, **kwargs):
+    """Shortcut function to invoke multi-replace in a single command."""
+    m = MultiReplace(sub_map, **kwargs)
+    return m.sub(text)
