@@ -1,6 +1,24 @@
 # -*- coding: utf-8 -*-
-"""
-TODO: docs
+""":mod:`urlutils` is a module dedicated to one of software's most
+versatile, well-aged, and beloved data structures: the URL, also known
+as the `Uniform Resource Locator`_.
+
+Among other things, this module is a full reimplementation of URLs,
+without any reliance on the :mod:`urlparse` or :mod:`urllib` standard
+library modules. The centerpiece and top-level interface of urlutils
+is the :class:`URL` type. Also featured is the :func:`find_all_links`
+convenience function. Some low-level functions and constants are also
+below.
+
+The implementations in this module are based heavily on `RFC 3986`_ and
+`RFC 3987`_, and incorporates details from several other RFCs and `W3C
+documents`_.
+
+.. _Uniform Resource Locator: https://en.wikipedia.org/wiki/Uniform_Resource_Locator
+.. _RFC 3986: https://tools.ietf.org/html/rfc3986
+.. _RFC 3987: https://tools.ietf.org/html/rfc3987
+.. _W3C documents: https://www.w3.org/TR/uri-clarification/
+
 """
 
 import re
@@ -14,15 +32,15 @@ try:
 except NameError:
     unichr = chr
 
-# The unreserved URI characters (per RFC 3986)
+# The unreserved URI characters (per RFC 3986 Section 2.3)
 _UNRESERVED_CHARS = frozenset('~-._0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
                               'abcdefghijklmnopqrstuvwxyz')
 
-# URL parsing regex (per RFC 3986)
+# URL parsing regex (based on RFC 3986 Appendix B, with modifications)
 _URL_RE = re.compile(r'^((?P<scheme>[^:/?#]+):)?'
-                     r'((?P<_uses_netloc>//)(?P<authority>[^/?#]*))?'
-                     r'(?P<path_parts>[^?#]*)'
-                     r'(\?(?P<_query>[^#]*))?'
+                     r'((?P<_netloc_sep>//)(?P<authority>[^/?#]*))?'
+                     r'(?P<path>[^?#]*)'
+                     r'(\?(?P<query>[^#]*))?'
                      r'(#(?P<fragment>.*))?')
 
 
@@ -32,23 +50,49 @@ _HEX_CHAR_MAP = dict([((a + b).encode('ascii'),
 _ASCII_RE = re.compile('([\x00-\x7f]+)')
 
 
+# This port list painstakingly curated by hand searching through
+# https://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml
+# and
+# https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xhtml
+SCHEME_PORT_MAP = {'acap': 674, 'afp': 548, 'dict': 2628, 'dns': 53,
+                   'file': None, 'ftp': 21, 'git': 9418, 'gopher': 70,
+                   'http': 80, 'https': 443, 'imap': 143, 'ipp': 631,
+                   'ipps': 631, 'irc': 194, 'ircs': 6697, 'ldap': 389,
+                   'ldaps': 636, 'mms': 1755, 'msrp': 2855, 'msrps': None,
+                   'mtqp': 1038, 'nfs': 111, 'nntp': 119, 'nntps': 563,
+                   'pop': 110, 'prospero': 1525, 'redis': 6379, 'rsync': 873,
+                   'rtsp': 554, 'rtsps': 322, 'rtspu': 5005, 'sftp': 22,
+                   'smb': 445, 'snmp': 161, 'ssh': 22, 'steam': None,
+                   'svn': 3690, 'telnet': 23, 'ventrilo': 3784, 'vnc': 5900,
+                   'wais': 210, 'ws': 80, 'wss': 443, 'xmpp': None}
+
+# This list of schemes that don't use authorities is also from the link above.
+NO_NETLOC_SCHEMES = set(['urn', 'about', 'bitcoin', 'blob', 'data', 'geo',
+                         'magnet', 'mailto', 'news', 'pkcs11',
+                         'sip', 'sips', 'tel'])
+# As of Mar 11, 2017, there were 44 netloc schemes, and 13 non-netloc
+
 # RFC 3986 section 2.2, Reserved Characters
 _GEN_DELIMS = frozenset(u':/?#[]@')
 _SUB_DELIMS = frozenset(u"!$&'()*+,;=")
 _ALL_DELIMS = _GEN_DELIMS | _SUB_DELIMS
 
 _USERINFO_SAFE = _UNRESERVED_CHARS | _SUB_DELIMS
+_USERINFO_DELIMS = _ALL_DELIMS - _USERINFO_SAFE
 _PATH_SAFE = _UNRESERVED_CHARS | _SUB_DELIMS | set(u':@')
+_PATH_DELIMS = _ALL_DELIMS - _PATH_SAFE
 _FRAGMENT_SAFE = _UNRESERVED_CHARS | _PATH_SAFE | set(u'/?')
+_FRAGMENT_DELIMS = _ALL_DELIMS - _FRAGMENT_SAFE
 _QUERY_SAFE = _UNRESERVED_CHARS | _FRAGMENT_SAFE - set(u'&=+')
+_QUERY_DELIMS = _ALL_DELIMS - _QUERY_SAFE
 
-NETLOC_SCHEMES = ['ftp', 'http', 'gopher', 'nntp', 'telnet',
-                  'imap', 'wais', 'file', 'mms', 'https', 'shttp',
-                  'snews', 'prospero', 'rtsp', 'rtspu', 'rsync', '',
-                  'svn', 'svn+ssh', 'sftp', 'nfs', 'git', 'git+ssh']
-NO_NETLOC_SCHEMES = ['urn', 'tel', 'news', 'mailto', 'magnet']  # TODO: others?
 
-DEFAULT_PORT_MAP = {'http': 80, 'https': 443}
+class URLParseError(ValueError):
+    """Exception inheriting from :exc:`ValueError`, raised when failing to
+    parse a URL. Mostly raised on invalid ports and IPv6 addresses.
+    """
+    pass
+
 
 DEFAULT_ENCODING = 'utf8'
 
@@ -60,19 +104,44 @@ def to_unicode(obj):
         return unicode(obj, encoding=DEFAULT_ENCODING)
 
 
-class URLParseError(ValueError):
-    pass
-
-
 # regex from gruber via tornado
-# doesn't support ipv6 or mailto (netloc-less schemes)
+# doesn't support ipv6
+# doesn't support mailto (netloc-less schemes)
 _FIND_ALL_URL_RE = re.compile(to_unicode(r"""\b((?:([\w-]+):(/{1,3})|www[.])(?:(?:(?:[^\s&()<>]|&amp;|&quot;)*(?:[^!"#$%'()*+,.:;<=>?@\[\]^`{|}~\s]))|(?:\((?:[^\s&()]|&amp;|&quot;)*\)))+)"""))
 
 
-def find_all_links(text, with_text=False,
-                   require_scheme=False, default_scheme='http', schemes=()):
-    """
-    Heuristic-based link finder.
+def find_all_links(text, with_text=False, default_scheme='https', schemes=()):
+    """This function uses heuristics to searches plain text for strings
+    that look like URLs, returning a :class:`list` of :class:`URL`
+    objects. It supports limiting the accepted schemes, and returning
+    interleaved text as well.
+
+    >>> find_all_links('Visit https://boltons.rtfd.org!')
+    [URL(u'https://boltons.rtfd.org')]
+    >>> find_all_links('Visit https://boltons.rtfd.org!', with_text=True)
+    [u'Visit ', URL(u'https://boltons.rtfd.org'), u'!']
+
+    Args:
+       text (str): The text to search.
+
+       with_text (bool): Whether or not to interleave plaintext blocks
+          with the returned URL objects. Having all tokens can be
+          useful for transforming the text, e.g., replacing links with
+          HTML equivalents. Defaults to ``False``.
+
+       default_scheme (str): Many URLs are written without the scheme
+          component. This function can match a reasonable subset of
+          those, provided *default_scheme* is set to a string. Set to
+          ``False`` to disable matching scheme-less URLs. Defaults to
+          ``'https'``.
+
+       schemes (list): A list of strings that a URL's scheme must
+          match in order to be included in the results. Defaults to
+          empty, which matches all schemes.
+
+    .. note:: Currently this function does not support finding IPv6
+      addresses or URLs with netloc-less schemes, like mailto.
+
     """
     text = to_unicode(text)
     prev_end, start, end = 0, None, None
@@ -94,10 +163,11 @@ def find_all_links(text, with_text=False,
             cur_url_text = match.group(0)
             cur_url = URL(cur_url_text)
             if not cur_url.scheme:
-                if require_scheme:
-                    _add_text(text[start:end])
-                else:
+                if default_scheme:
                     cur_url = URL(default_scheme + '://' + cur_url_text)
+                else:
+                    _add_text(text[start:end])
+                    continue
             if schemes and cur_url.scheme not in schemes:
                 _add_text(text[start:end])
             else:
@@ -136,50 +206,61 @@ _FRAGMENT_QUOTE_MAP = _make_quote_map(_FRAGMENT_SAFE)
 
 
 def quote_path_part(text, full_quote=True):
-    # TODO: why does one route allow percents through and not the
-    # other?
-    if not full_quote:
-        return u''.join([_PATH_PART_QUOTE_MAP.get(t, t) for t in text])
-
-    bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
-    return u''.join([_PATH_PART_QUOTE_MAP[b] for b in bytestr])
+    """
+    Percent-encode a single segment of a URL path.
+    """
+    if full_quote:
+        bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
+        return u''.join([_PATH_PART_QUOTE_MAP[b] for b in bytestr])
+    return u''.join([_PATH_PART_QUOTE_MAP[t] if t in _PATH_DELIMS else t
+                     for t in text])
 
 
 def quote_query_part(text, full_quote=True):
-    if not full_quote:
-        return u''.join([_QUERY_PART_QUOTE_MAP.get(t, t) for t in text])
-
-    bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
-    return u''.join([_QUERY_PART_QUOTE_MAP[b] for b in bytestr])
+    """
+    Percent-encode a single query string key or value.
+    """
+    if full_quote:
+        bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
+        return u''.join([_QUERY_PART_QUOTE_MAP[b] for b in bytestr])
+    return u''.join([_QUERY_PART_QUOTE_MAP[t] if t in _QUERY_DELIMS else t
+                     for t in text])
 
 
-# fragments don't really have parts, there are no official
-# subdelimiters within fragments, I believe
 def quote_fragment_part(text, full_quote=True):
-    if not full_quote:
-        return u''.join([_FRAGMENT_QUOTE_MAP.get(t, t) for t in text])
-
-    bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
-    return u''.join([_FRAGMENT_QUOTE_MAP[b] for b in bytestr])
+    """Quote the fragment part of the URL. Fragments don't have
+    subdelimiters, so the whole URL fragment can be passed.
+    """
+    if full_quote:
+        bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
+        return u''.join([_FRAGMENT_QUOTE_MAP[b] for b in bytestr])
+    return u''.join([_FRAGMENT_QUOTE_MAP[t] if t in _FRAGMENT_DELIMS else t
+                     for t in text])
 
 
 def quote_userinfo_part(text, full_quote=True):
-    if not full_quote:
-        return u''.join([_USERINFO_PART_QUOTE_MAP.get(t, t) for t in text])
-
-    bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
-    return u''.join([_USERINFO_PART_QUOTE_MAP[b] for b in bytestr])
+    """Quote special characters in either the username or password
+    section of the URL. Note that userinfo in URLs is considered
+    deprecated in many circles (especially browsers), and support for
+    percent-encoded userinfo can be spotty.
+    """
+    if full_quote:
+        bytestr = normalize('NFC', to_unicode(text)).encode('utf8')
+        return u''.join([_USERINFO_PART_QUOTE_MAP[b] for b in bytestr])
+    return u''.join([_USERINFO_PART_QUOTE_MAP[t] if t in _USERINFO_DELIMS
+                     else t for t in text])
 
 
 def unquote(string, encoding='utf-8', errors='replace'):
-    """Replace %xx escapes by their single-character equivalent. The optional
-    encoding and errors parameters specify how to decode percent-encoded
-    sequences into Unicode characters, as accepted by the bytes.decode()
-    method.
-    By default, percent-encoded sequences are decoded with UTF-8, and invalid
-    sequences are replaced by a placeholder character.
+    """Percent-decode a string, by replacing %xx escapes with their
+    single-character equivalent. The optional *encoding* and *errors*
+    parameters specify how to decode percent-encoded sequences into
+    Unicode characters, as accepted by the :meth:`bytes.decode()` method.  By
+    default, percent-encoded sequences are decoded with UTF-8, and
+    invalid sequences are replaced by a placeholder character.
 
-    unquote('abc%20def') -> 'abc def'.
+    >>> unquote(u'abc%20def')
+    u'abc def'
     """
     if '%' not in string:
         string.split
@@ -226,7 +307,21 @@ def unquote_to_bytes(string):
 
 def register_scheme(text, uses_netloc=None, default_port=None):
     """Registers new scheme information, resulting in correct port and
-    slash behavior from the URL object.
+    slash behavior from the URL object. There are dozens of standard
+    schemes preregistered, so this function is mostly meant for
+    proprietary internal customizations or stopgaps on missing
+    standards information. If a scheme seems to be missing, please
+    `file an issue`_!
+
+    Args:
+        text (str): Text representing the scheme.
+           (the 'http' in 'http://hatnote.com')
+        uses_netloc (bool): Does the scheme support specifying a
+           network host? For instance, "http" does, "mailto" does not.
+        default_port (int): The default port, if any, for netloc-using
+           schemes.
+
+    .. _file an issue: https://github.com/mahmoud/boltons/issues
     """
     text = text.lower()
     if default_port is not None:
@@ -237,23 +332,22 @@ def register_scheme(text, uses_netloc=None, default_port=None):
                              % (default_port,))
 
     if uses_netloc is True:
-        if text not in NETLOC_SCHEMES:
-            NETLOC_SCHEMES.append(text)
+        SCHEME_PORT_MAP[text] = default_port
     elif uses_netloc is False:
-        if text not in NO_NETLOC_SCHEMES:
-            NO_NETLOC_SCHEMES.append(text)
+        if default_port is not None:
+            raise ValueError('unexpected default port while specifying'
+                             ' non-netloc scheme: %r' % default_port)
+        NO_NETLOC_SCHEMES.add(text)
     elif uses_netloc is not None:
         raise ValueError('uses_netloc expected True, False, or None')
 
-    DEFAULT_PORT_MAP[text] = default_port
-
     return
 
 
 def resolve_path_parts(path_parts):
-    """
-    Normalize the URL path by resolving segments of '.' and '..'.
-    See RFC 3986 section 5.2.4, Remove Dot Segments.
+    """Normalize the URL path by resolving segments of '.' and '..',
+    resulting in a dot-free path.  See RFC 3986 section 5.2.4, Remove
+    Dot Segments.
     """
     # TODO: what to do with multiple slashes
     ret = []
@@ -299,15 +393,66 @@ class cachedproperty(object):
 
 
 class URL(object):
+    """The URL is one of the most ubiquitous data structures in the
+    virtual and physical landscape. From blogs to billboards, URLs are
+    so common, that it's easy to overlook their complexity and
+    power.
 
-    _attrs = ('scheme', '_uses_netloc', 'username', 'password', 'family',
-              'host', 'port', 'path_parts', '_query', 'fragment')
+    There are 8 parts of a URL, each with its own semantics and
+    special characters:
+
+      * :attr:`~URL.scheme`
+      * :attr:`~URL.username`
+      * :attr:`~URL.password`
+      * :attr:`~URL.host`
+      * :attr:`~URL.port`
+      * :attr:`~URL.path`
+      * :attr:`~URL.query_params` (query string parameters)
+      * :attr:`~URL.fragment`
+
+    Each is exposed as an attribute on the URL object. RFC 3986 offers
+    this brief structural summary of the main URL components::
+
+        foo://user:pass@example.com:8042/over/there?name=ferret#nose
+        \_/   \_______/ \_________/ \__/\_________/ \_________/ \__/
+         |        |          |        |      |           |        |
+       scheme  userinfo     host     port   path       query   fragment
+
+    And here's how that example can be manipulated with the URL type:
+
+    >>> url = URL('foo://example.com:8042/over/there?name=ferret#nose')
+    >>> print(url.host)
+    example.com
+    >>> print(url.get_authority())
+    example.com:8042
+    >>> print(url.qp['name'])  # qp is a synonym for query_params
+    ferret
+
+    URL's approach to encoding is that inputs are decoded as much as
+    possible, and data remains in this decoded state until re-encoded
+    using the :meth:`~URL.to_text()` method. In this way, it's similar
+    to Python's current approach of encouraging immediate decoding of
+    bytes to text.
+
+    Note that URL instances are mutable objects. If an immutable
+    representation of the URL is desired, the string from
+    :meth:`~URL.to_text()` may be used. For an immutable, but
+    almost-as-featureful, URL object, check out the `hyperlink
+    package`_.
+
+    .. _hyperlink package: https://github.com/mahmoud/hyperlink
+
+    """
+
+    # public attributes (for comparison, see __eq__):
+    _cmp_attrs = ('scheme', 'uses_netloc', 'username', 'password',
+                  'family', 'host', 'port', 'path', 'query_params', 'fragment')
 
     def __init__(self, url=''):
         # TODO: encoding param. The encoding that underlies the
         # percent-encoding is always utf8 for IRIs, but can be Latin-1
         # for other usage schemes.
-        url_dict = {}
+        ud = DEFAULT_PARSED_URL
         if url:
             if isinstance(url, URL):
                 url = url.to_text()  # better way to copy URLs?
@@ -319,32 +464,59 @@ class URL(object):
                                         ' try decoding the url bytes and'
                                         ' passing the result. (got: %s)'
                                         % (DEFAULT_ENCODING, ude))
-            url_dict = parse_url(url)
+            ud = parse_url(url)
 
-        _d = unicode()
-        for attr in self._attrs:
-            # TODO: possibly use None as marker for empty vs missing
-            val = url_dict.get(attr, _d) or _d
-            if attr == 'path_parts':
-                val = tuple([unquote(p) if '%' in p else p
-                             for p in val.split(u'/')])
-            elif attr in ('username', 'password', 'fragment') and '%' in val:
-                val = unquote(val)
-            elif attr == 'host' and val:
-                try:
-                    val = val.encode("ascii")
-                except UnicodeEncodeError:
-                    pass  # already non-ascii text
-                else:
-                    val = val.decode("idna")
-            setattr(self, attr, val)
+        _e = u''
+        self.scheme = ud['scheme'] or _e
+        self._netloc_sep = ud['_netloc_sep'] or _e
+        self.username = (unquote(ud['username'])
+                         if '%' in (ud['username'] or _e) else ud['username'] or _e)
+        self.password = (unquote(ud['password'])
+                         if '%' in (ud['password'] or _e) else ud['password'] or _e)
+        self.family = ud['family']
+
+        if not ud['host']:
+            self.host = _e
+        else:
+            try:
+                self.host = ud['host'].encode("ascii")
+            except UnicodeEncodeError:
+                self.host = ud['host']  # already non-ascii text
+            else:
+                self.host = self.host.decode("idna")
+
+        self.port = ud['port']
+        self.path_parts = tuple([unquote(p) if '%' in p else p for p
+                                 in (ud['path'] or _e).split(u'/')])
+        self._query = ud['query'] or _e
+        self.fragment = (unquote(ud['fragment'])
+                         if '%' in (ud['fragment'] or _e) else ud['fragment'] or _e)
+        # TODO: possibly use None as marker for empty vs missing
         return
 
     @classmethod
     def from_parts(cls, scheme=None, host=None, path_parts=(), query_params=(),
                    fragment=u'', port=None, username=None, password=None):
-        """
-        Build a new URL from parts.
+        """Build a new URL from parts. Note that the respective arguments are
+        not in the order they would appear in a URL:
+
+        Args:
+           scheme (str): The scheme of a URL, e.g., 'http'
+           host (str): The host string, e.g., 'hatnote.com'
+           path_parts (tuple): The individual text segments of the
+             path, e.g., ('post', '123')
+           query_params (dict): An OMD, dict, or list of (key, value)
+             pairs representing the keys and values of the URL's query
+             parameters.
+           fragment (str): The fragment of the URL, e.g., 'anchor1'
+           port (int): The integer port of URL, automatic defaults are
+             available for registered schemes.
+           username (str): The username for the userinfo part of the URL.
+           password (str): The password for the userinfo part of the URL.
+
+        Note that this method does relatively little
+        validation. :meth:`URL.to_text()` should be used to check if
+        any errors are produced while composing the final textual URL.
         """
         ret = cls()
 
@@ -361,12 +533,21 @@ class URL(object):
 
     @cachedproperty
     def query_params(self):
+        """The parsed form of the query string of the URL, represented as a
+        :class:`~dictutils.OrderedMultiDict`. Also available as the
+        handy alias ``qp``.
+
+        >>> url = URL('http://boltons.readthedocs.io/?utm_source=doctest&python=great')
+        >>> url.qp.keys()
+        [u'utm_source', u'python']
+        """
         return QueryParamDict.from_text(self._query)
 
     qp = query_params
 
     @property
     def path(self):
+        "The URL's path, in text form."
         return u'/'.join([quote_path_part(p, full_quote=False)
                           for p in self.path_parts])
 
@@ -378,28 +559,62 @@ class URL(object):
 
     @property
     def uses_netloc(self):
-        default = self._uses_netloc
-        if self.scheme in NETLOC_SCHEMES:
+        """Whether or not a URL uses :code:`:` or :code:`://` to separate the
+        scheme from the rest of the URL depends on the scheme's own
+        standard definition. There is no way to infer this behavior
+        from other parts of the URL. A scheme either supports network
+        locations or it does not.
+
+        The URL type's approach to this is to check for explicitly
+        registered schemes, with common schemes like HTTP
+        preregistered. This is the same approach taken by
+        :mod:`urlparse`.
+
+        URL adds two additional heuristics if the scheme as a whole is
+        not registered. First, it attempts to check the subpart of the
+        scheme after the last ``+`` character. This adds intuitive
+        behavior for schemes like ``git+ssh``. Second, if a URL with
+        an unrecognized scheme is loaded, it will maintain the
+        separator it sees.
+
+        >>> print(URL('fakescheme://test.com').to_text())
+        fakescheme://test.com
+        >>> print(URL('mockscheme:hello:world').to_text())
+        mockscheme:hello:world
+
+        """
+        default = self._netloc_sep
+        if self.scheme in SCHEME_PORT_MAP:
             return True
         if self.scheme in NO_NETLOC_SCHEMES:
             return False
-        if self.scheme.split('+')[-1] in NETLOC_SCHEMES:
+        if self.scheme.split('+')[-1] in SCHEME_PORT_MAP:
             return True
         return default
 
     @property
     def default_port(self):
+        """Return the default port for the currently-set scheme. Returns
+        ``None`` if the scheme is unrecognized. See
+        :func:`register_scheme` above. If :attr:`~URL.port` matches
+        this value, no port is emitted in the output of
+        :meth:`~URL.to_text()`.
+
+        Applies the same '+' heuristic detailed in :meth:`URL.uses_netloc`.
+        """
         try:
-            return DEFAULT_PORT_MAP[self.scheme]
+            return SCHEME_PORT_MAP[self.scheme]
         except KeyError:
-            return DEFAULT_PORT_MAP.get(self.scheme.split('+')[-1])
+            return SCHEME_PORT_MAP.get(self.scheme.split('+')[-1])
 
     def normalize(self, with_case=True):
         """Resolve any "." and ".." references in the path, as well as
-        normalize scheme and host casing.
+        normalize scheme and host casing. To turn off case
+        normalization, pass ``with_case=False``.
 
-        More information can be found in Section 6.2.2 of RFC 3986.
+        More information can be found in `Section 6.2.2 of RFC 3986`_.
 
+        .. _Section 6.2.2 of RFC 3986: https://tools.ietf.org/html/rfc3986#section-6.2.2
         """
         self.path_parts = resolve_path_parts(self.path_parts)
 
@@ -409,8 +624,22 @@ class URL(object):
         return
 
     def navigate(self, dest):
-        """Factory method that returns a _new_ URL based on a given
-        destination, *dest*.
+        """Factory method that returns a _new_ :class:`URL` based on a given
+        destination, *dest*. Useful for navigating those relative
+        links with ease.
+
+        The newly created :class:`URL` is normalized before being returned.
+
+        >>> url = URL('http://boltons.readthedocs.io')
+        >>> url.navigate('en/latest/')
+        URL(u'http://boltons.readthedocs.io/en/latest/')
+
+        Args:
+           dest (str): A string or URL object representing the destination
+
+        More information can be found in `Section 5 of RFC 3986`_.
+
+        .. _Section 5 of RFC 3986: https://tools.ietf.org/html/rfc3986#section-5
         """
         orig_dest = None
         if not isinstance(dest, URL):
@@ -442,13 +671,36 @@ class URL(object):
         ret.normalize()
         return ret
 
-    def get_authority(self, full_quote=True, with_userinfo=True):
+    def get_authority(self, full_quote=False, with_userinfo=False):
+        """Used by URL schemes that have a network location,
+        :meth:`~URL.get_authority` combines :attr:`username`,
+        :attr:`password`, :attr:`host`, and :attr:`port` into one
+        string, the *authority*, that is used for
+        connecting to a network-accessible resource.
+
+        Used internally by :meth:`~URL.to_text()` and can be useful
+        for labeling connections.
+
+        >>> url = URL('ftp://user@ftp.debian.org:2121/debian/README')
+        >>> print(url.get_authority())
+        ftp.debian.org:2121
+        >>> print(url.get_authority(with_userinfo=True))
+        user@ftp.debian.org:2121
+
+        Args:
+           full_quote (bool): Whether or not to apply IDNA encoding.
+              Defaults to ``False``.
+           with_userinfo (bool): Whether or not to include username
+              and password, technically part of the
+              authority. Defaults to ``False``.
+
+        """
         parts = []
         _add = parts.append
         if self.username and with_userinfo:
             _add(quote_userinfo_part(self.username))
-            _add(':')
             if self.password:
+                _add(':')
                 _add(quote_userinfo_part(self.password))
             _add('@')
         if self.host:
@@ -467,10 +719,26 @@ class URL(object):
         return u''.join(parts)
 
     def to_text(self, full_quote=False):
+        """Render a string representing the current state of the URL
+        object.
+
+        >>> url = URL('http://listen.hatnote.com')
+        >>> url.fragment = 'en'
+        >>> print(url.to_text())
+        http://listen.hatnote.com#en
+
+        By setting the *full_quote* flag, the URL can either be fully
+        quoted or minimally quoted. The most common characteristic of
+        an encoded-URL is the presence of percent-encoded text (e.g.,
+        %60).  Unquoted URLs are more readable and suitable
+        for display, whereas fully-quoted URLs are more conservative
+        and generally necessary for sending over the network.
+        """
         scheme = self.scheme
         path = u'/'.join([quote_path_part(p, full_quote=full_quote)
                           for p in self.path_parts])
-        authority = self.get_authority(full_quote=full_quote)
+        authority = self.get_authority(full_quote=full_quote,
+                                       with_userinfo=True)
         query_string = self.query_params.to_text(full_quote=full_quote)
         fragment = quote_fragment_part(self.fragment, full_quote=full_quote)
 
@@ -503,7 +771,7 @@ class URL(object):
         return u'%s(%r)' % (cn, self.to_text())
 
     def __eq__(self, other):
-        for attr in self._attrs:
+        for attr in self._cmp_attrs:
             if not getattr(self, attr) == getattr(other, attr, None):
                 return False
         return True
@@ -530,7 +798,7 @@ except ImportError:
 
     def inet_pton(address_family, ip_string):
         addr = _sockaddr()
-        ip_string = ip_string.encode('ascii')  # TODO
+        ip_string = ip_string.encode('ascii')
         addr.sa_family = address_family
         addr_size = ctypes.c_int(ctypes.sizeof(addr))
 
@@ -546,8 +814,10 @@ except ImportError:
 
 def parse_host(host):
     """\
-    returns:
-      family (socket constant or None), host (string)
+    Low-level function used to parse the host portion of a URL.
+
+    Returns a tuple of (family, host) where *family* is a
+    :mod:`socket` module constant or ``None``, and host is a string.
 
     >>> parse_host('googlewebsite.com') == (None, 'googlewebsite.com')
     True
@@ -556,8 +826,9 @@ def parse_host(host):
     >>> parse_host('192.168.1.1') == (socket.AF_INET, '192.168.1.1')
     True
 
-    (odd doctest formatting above due to py3's switch from int to enums
-    for socket constants)
+    Odd doctest formatting above due to py3's switch from int to enums
+    for :mod:`socket` constants.
+
     """
     if not host:
         return None, u''
@@ -582,6 +853,24 @@ def parse_host(host):
 
 
 def parse_url(url_text):
+    """\
+    Used to parse the text for a single URL into a dictionary, used
+    internally by the :class:`URL` type.
+
+    Note that "URL" has a very narrow, standards-based
+    definition. While :func:`parse_url` may raise
+    :class:`URLParseError` under a very limited number of conditions,
+    such as non-integer port, a surprising number of strings are
+    technically valid URLs. For instance, the text ``"url"`` is a
+    valid URL, because it is a relative path.
+
+    In short, do not expect this function to validate form inputs or
+    other more colloquial usages of URLs.
+
+    >>> res = parse_url('http://127.0.0.1:3000/?a=1')
+    >>> sorted(res.keys())  # res is a basic dictionary
+    ['_netloc_sep', 'authority', 'family', 'fragment', 'host', 'password', 'path', 'port', 'query', 'scheme', 'username']
+    """
     url_text = unicode(url_text)
     # raise TypeError('parse_url expected text, not %r' % url_str)
     um = _URL_RE.match(url_text)
@@ -624,7 +913,13 @@ def parse_url(url_text):
     return gs
 
 
+DEFAULT_PARSED_URL = parse_url('')
+
+
 def parse_qsl(qs, keep_blank_values=True, encoding=DEFAULT_ENCODING):
+    """
+    Converts a query string into a list of (key, value) pairs.
+    """
     pairs = [s2 for s1 in qs.split('&') for s2 in s1.split(';')]
     ret = []
     for pair in pairs:
@@ -1207,15 +1502,41 @@ OMD = OrderedMultiDict
 
 
 class QueryParamDict(OrderedMultiDict):
-    # TODO: caching
-    # TODO: self.update_extend_from_text()?
+    """A subclass of :class:`~dictutils.OrderedMultiDict` specialized for
+    representing query string values. Everything is fully unquoted on
+    load and all parsed keys and values are strings by default.
+
+    As the name suggests, multiple values are supported and insertion
+    order is preserved.
+
+    >>> qp = QueryParamDict.from_text(u'key=val1&key=val2&utm_source=rtd')
+    >>> qp.getlist('key')
+    [u'val1', u'val2']
+    >>> qp['key']
+    u'val2'
+    >>> qp.add('key', 'val3')
+    >>> qp.to_text()
+    'key=val1&key=val2&utm_source=rtd&key=val3'
+
+    See :class:`~dictutils.OrderedMultiDict` for more API features.
+    """
 
     @classmethod
     def from_text(cls, query_string):
+        """
+        Parse *query_string* and return a new :class:`QueryParamDict`.
+        """
         pairs = parse_qsl(query_string, keep_blank_values=True)
         return cls(pairs)
 
     def to_text(self, full_quote=False):
+        """
+        Render and return a query string.
+
+        Args:
+           full_quote (bool): Whether or not to percent-quote special
+              characters or leave them decoded for readability.
+        """
         ret_list = []
         for k, v in self.iteritems(multi=True):
             key = quote_query_part(to_unicode(k), full_quote=full_quote)
@@ -1226,5 +1547,6 @@ class QueryParamDict(OrderedMultiDict):
                 ret_list.append(u'='.join((key, val)))
         return u'&'.join(ret_list)
 
+# TODO: cleanup OMD/cachedproperty etc.?
 
 # end urlutils.py
