@@ -1,14 +1,18 @@
 
+import os
+
 import pytest
 
 from boltons.dictutils import OMD
 from boltons.iterutils import (first,
                                remap,
+                               research,
                                default_enter,
                                default_exit,
                                get_path)
 from boltons.namedutils import namedtuple
 
+CUR_PATH = os.path.abspath(__file__)
 
 isbool = lambda x: isinstance(x, bool)
 isint = lambda x: isinstance(x, int)
@@ -339,6 +343,18 @@ class TestRemap(object):
         fs = frozenset([1, 2, 3])
         assert remap(fs) == fs
 
+    def test_remap_file(self):
+        with open(CUR_PATH, 'rb') as f:
+            x = {'a': [1, 2, 3], 'f': [f]}
+            assert remap(x) == x
+
+            f.read()
+            assert remap(x) == x
+
+            f.close()  # see #146
+            assert remap(x) == x
+        return
+
 
 class TestGetPath(object):
     def test_depth_one(self):
@@ -356,6 +372,27 @@ class TestGetPath(object):
         assert get_path(root, 'key.0') == 'test'
 
 
+def test_research():
+    root = {}
+
+    with pytest.raises(TypeError):
+        research(root, query=None)
+
+    root = {'a': 'a'}
+    res = research(root, query=lambda p, k, v: v == 'a')
+    assert len(res) == 1
+    assert res[0] == (('a',), 'a')
+
+    def broken_query(p, k, v):
+        raise RuntimeError()
+
+    with pytest.raises(RuntimeError):
+        research(root, broken_query, reraise=True)
+
+    # empty results with default, reraise=False
+    assert research(root, broken_query) == []
+
+
 def test_backoff_basic():
     from boltons.iterutils import backoff
 
@@ -442,8 +479,8 @@ def test_guiderator():
     assert guid != guid2
 
     # custom size
-    guid_iter = GUIDerator(size=16)
-    assert len(next(guid_iter)) == 16
+    guid_iter = GUIDerator(size=26)
+    assert len(next(guid_iter)) == 26
 
 
 def test_seqguiderator():
@@ -463,5 +500,5 @@ def test_seqguiderator():
 
     # custom size
     for x in range(10000):
-        guid_iter = GUIDerator(size=16)
-        assert len(next(guid_iter)) == 16
+        guid_iter = GUIDerator(size=26)
+        assert len(next(guid_iter)) == 26
